/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "Map.h"

using namespace std;
using namespace FD;

class CvfdhGetQuadrangleSubPix;
   
DECLARE_NODE(CvfdhGetQuadrangleSubPix)
/*Node
 *
 * @name CvfdhGetQuadrangleSubPix
 * @category FDOPENCV_HARK:Transform
 * @description CvfdhGetQuadrangleSubPix
 *
 * @input_name IMAGEIN
 * @input_type any
 * @input_description Input image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @output_name IMAGEOUT
 * @output_type any
 * @output_description Output image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @parameter_name ROTATION_CENTER
 * @parameter_type string
 * @parameter_value CENTER
 * @parameter_list CENTER:OWN_SETTING
 * @parameter_description Center of the rotation
 * 
 * @parameter_name CENTER_X
 * @parameter_type float
 * @parameter_value 200
 * @parameter_valid ROTATION_CENTER=OWN_SETTING
 * @parameter_description x-position of the rotation center
 * 
 * @parameter_name CENTER_Y
 * @parameter_type float
 * @parameter_value 200
 * @parameter_valid ROTATION_CENTER=OWN_SETTING
 * @parameter_description y-position of the rotation center
 * 
 * @parameter_name ANGLE
 * @parameter_type float
 * @parameter_value 45
 * @parameter_description Rotation angle in degree
 *
 END*/
   
   
class CvfdhGetQuadrangleSubPix : public BufferedNode {

  int m_imageInID;
  int m_imageOutID;

  string rotation_center;
  
  float center_x;
  float center_y;
  float center_x_own;
  float center_y_own;
  float angle;
      
public:
  CvfdhGetQuadrangleSubPix(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {

    m_imageInID = addInput("IMAGEIN");
    m_imageOutID = addOutput("IMAGEOUT");

    rotation_center = object_cast<string>(parameters.get("ROTATION_CENTER"));
    
    center_x = 0.0;
    center_y = 0.0;
    center_x_own = 0.0;
    center_y_own = 0.0;

    if(parameters.exist("CENTER_X") == true)
      center_x_own = dereference_cast<float>(parameters.get("CENTER_X"));
    
    if(parameters.exist("CENTER_Y") == true)
      center_y_own = dereference_cast<float>(parameters.get("CENTER_Y"));

    angle    = dereference_cast<float>(parameters.get("ANGLE"));    

    inOrder = true;
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {

    ObjectRef in = getInput(m_imageInID, count);

    // -------------------------------------------------------------

    if (typeid(*in) == typeid(CvfdImage)) {

      RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   
      if(imagePtr->getImage() == NULL){
	(*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr);
	return;
      }

      IplImage* img_in;
      img_in = cvCloneImage(imagePtr->getImage());
    
      float m[6];
      IplImage *dst_img = 0;
      CvMat M;
    
      dst_img = cvCloneImage (img_in);

      if(rotation_center == "CENTER"){
	center_x = img_in->width * 0.5;
	center_y = img_in->height * 0.5;
      }else if(rotation_center == "OWN_SETTING"){
	center_x = center_x_own;
	center_y = center_y_own;
      }
  
      m[0] = (float) (cos (angle * CV_PI / 180.));
      m[1] = (float) (-sin (angle * CV_PI / 180.));
      m[2] = center_x;
      m[3] = -m[1];
      m[4] = m[0];
      m[5] = center_y;
      cvInitMatHeader (&M, 2, 3, CV_32FC1, m, CV_AUTOSTEP);
    
      cvGetQuadrangleSubPix (img_in, dst_img, &M);

      IplImage* img_out;
      img_out = cvCloneImage(dst_img);

      CvfdImage* image= new CvfdImage(img_out);  
         
      (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(image); 

      cvReleaseImage (&img_in);
      cvReleaseImage (&img_out);
      cvReleaseImage (&dst_img);

      // -------------------------------------------------------------

    }else if (typeid(*in) == typeid(Map<int, ObjectRef>)) {

      RCPtr<Map<int, ObjectRef> > imageMapPtr = getInput(m_imageInID, count);
      RCPtr<Map<int, ObjectRef> > outputMapPtr(new Map<int, ObjectRef>);
      
      Map<int, ObjectRef>::iterator it = imageMapPtr->begin();
      for (; it != imageMapPtr->end(); ++it) {
        CvfdImage& imagePtr = object_cast<CvfdImage>(it->second);
	IplImage* img_in;
	img_in = cvCloneImage(imagePtr.getImage());

	float m[6];
	IplImage *dst_img = 0;
	CvMat M;
    
	dst_img = cvCloneImage (img_in);

	if(rotation_center == "CENTER"){
	  center_x = img_in->width * 0.5;
	  center_y = img_in->height * 0.5;
	}else if(rotation_center == "OWN_SETTING"){
	  center_x = center_x_own;
	  center_y = center_y_own;
	}
  
	m[0] = (float) (cos (angle * CV_PI / 180.));
	m[1] = (float) (-sin (angle * CV_PI / 180.));
	m[2] = center_x;
	m[3] = -m[1];
	m[4] = m[0];
	m[5] = center_y;
	cvInitMatHeader (&M, 2, 3, CV_32FC1, m, CV_AUTOSTEP);
    
	cvGetQuadrangleSubPix (img_in, dst_img, &M);

	IplImage* img_out;
	img_out = cvCloneImage(dst_img);

	CvfdImage* image= new CvfdImage(img_out);  

	(*outputMapPtr)[it->first] = image;

	cvReleaseImage (&img_in);
	cvReleaseImage (&img_out);
	cvReleaseImage (&dst_img);

      }
      (*(outputs[m_imageOutID].buffer))[count] = outputMapPtr; 
    }

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhGetQuadrangleSubPix)
};
