/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "SrcInfo.h"
#include "Vector.h"
#include "Map.h"
#include <vector>

using namespace std;
using namespace FD;

class CvfdhMatchTemplate;
   
DECLARE_NODE(CvfdhMatchTemplate)
/*Node
 *
 * @name CvfdhMatchTemplate
 * @category FDOPENCV_HARK:Segmentation
 * @description CvfdhMatchTemplate
 *
 * @input_name IMAGEIN1
 * @input_type any
 * @input_description Input image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @input_name IMAGEIN2
 * @input_type any
 * @input_description Input image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @output_name SRCOUT
 * @output_type Vector<ObjectRef>
 * @output_description Source location. A type of an element of the vector is SrcInfo.
 *
 END*/
   
   
class CvfdhMatchTemplate : public BufferedNode {

  int m_imageIn1ID;
  int m_imageIn2ID;
  int m_srcOutID;
      
public:
  CvfdhMatchTemplate(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageIn1ID = addInput("IMAGEIN1");
    m_imageIn2ID = addInput("IMAGEIN2");
    m_srcOutID = addOutput("SRCOUT");
    inOrder = true;
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {

    Vector<ObjectRef>& sources = *new Vector<ObjectRef>;
    (*(outputs[m_srcOutID].buffer))[count] = &sources;

    vector<IplImage*> image_vec1;
    image_vec1.resize(0);

    vector<IplImage*> image_vec2;
    image_vec2.resize(0);

    IplImage* img_in_tmp1;
    ObjectRef in1 = getInput(m_imageIn1ID, count);
    if (typeid(*in1) == typeid(CvfdImage)) {
      RCPtr<CvfdImage> imagePtr1 = getInput(m_imageIn1ID,count);   
      if(imagePtr1->getImage() == NULL){
	return;
      }
      img_in_tmp1 = cvCloneImage(imagePtr1->getImage());
      image_vec1.push_back(img_in_tmp1);
    }else if (typeid(*in1) == typeid(Map<int, ObjectRef>)) {
      RCPtr<Map<int, ObjectRef> > imageMapPtr1 = getInput(m_imageIn1ID, count);
      Map<int, ObjectRef>::iterator it = imageMapPtr1->begin();
      for (; it != imageMapPtr1->end(); ++it) {
        CvfdImage& imagePtr1 = object_cast<CvfdImage>(it->second);
	img_in_tmp1 = cvCloneImage(imagePtr1.getImage());
	image_vec1.push_back(img_in_tmp1);
      }
    }

    IplImage* img_in_tmp2;
    ObjectRef in2 = getInput(m_imageIn2ID, count);
    if (typeid(*in2) == typeid(CvfdImage)) {
      RCPtr<CvfdImage> imagePtr2 = getInput(m_imageIn2ID,count);   
      if(imagePtr2->getImage() == NULL){
	return;
      }
      img_in_tmp2 = cvCloneImage(imagePtr2->getImage());
      image_vec2.push_back(img_in_tmp2);
    }else if (typeid(*in2) == typeid(Map<int, ObjectRef>)) {
      RCPtr<Map<int, ObjectRef> > imageMapPtr2 = getInput(m_imageIn2ID, count);
      Map<int, ObjectRef>::iterator it = imageMapPtr2->begin();
      for (; it != imageMapPtr2->end(); ++it) {
        CvfdImage& imagePtr2 = object_cast<CvfdImage>(it->second);
	img_in_tmp2 = cvCloneImage(imagePtr2.getImage());
	image_vec2.push_back(img_in_tmp2);
      }
    }

    for(int cnt = 0; cnt < image_vec1.size(); cnt++){

      IplImage* cv_image;      
      cv_image = cvCloneImage((IplImage*)image_vec1[cnt]);

      IplImage* tmp_img;      
      tmp_img = cvCloneImage((IplImage*)image_vec2[cnt]);
    
      // ---------------------------------------------------------------
      // main routine : start

      double min_val, max_val;
      CvPoint min_loc, max_loc;
      CvSize dst_size;
      IplImage *dst_img;
    
      dst_size = cvSize (cv_image->width - tmp_img->width + 1, cv_image->height - tmp_img->height + 1);
      dst_img = cvCreateImage (dst_size, IPL_DEPTH_32F, 1);
      cvMatchTemplate (cv_image, tmp_img, dst_img, CV_TM_CCOEFF_NORMED);
      cvMinMaxLoc (dst_img, &min_val, &max_val, &min_loc, &max_loc, NULL);
    
      cvRectangle (cv_image, max_loc, cvPoint (max_loc.x + tmp_img->width, max_loc.y + tmp_img->height), CV_RGB (255, 0, 0), 3);

      RCPtr<SrcInfo> src(new SrcInfo);
      src->roi_c[0] = (int)(max_loc.x + tmp_img->width / 2);
      src->roi_c[1] = (int)(max_loc.y + tmp_img->height / 2);
      src->roi_xy[0] = tmp_img->width;
      src->roi_xy[1] = tmp_img->height;
      src->img_size[0] = cv_image->width;
      src->img_size[1] = cv_image->height;
	
      sources.push_back(src);

      cvReleaseImage (&cv_image);
      cvReleaseImage (&tmp_img);
      cvReleaseImage (&dst_img);

    }

    cvReleaseImage(&img_in_tmp1);
    cvReleaseImage(&img_in_tmp2);

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhMatchTemplate)
};
