/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdStructuringElement.h"

using namespace std;
using namespace FD;

class CvfdCreateStructuringElement;
   
DECLARE_NODE(CvfdCreateStructuringElement)
/*Node
 *
 * @name CvfdCreateStructuringElement
 * @category FDOPENCV_ORG:Structuring Element
 * @description Create A Structuring Element FROM PARAMETERS
 * It allocates and fills the structure, which can be used as a structuring element in the morphological operations.
 *
 * @parameter_name COLS
 * @parameter_type int
 * @parameter_value 3
 * @parameter_description Number of columns in the structuring element
 *
 * @parameter_name ROWS
 * @parameter_type int
 * @parameter_value 3
 * @parameter_description Number of rows in the structuring element
 *
 * @parameter_name ANCHOR_X
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Relative horizontal offset of the anchor point
 *
 * @parameter_name ANCHORY_y
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Relative vertical offset of the anchor point 
 *
 * @parameter_name SHAPE
 * @parameter_type string
 * @parameter_value CV_SHAPE_RECT;CV_SHAPE_CROSS;CV_SHAPE_ELLIPSE
 * @parameter_description Shape of the structuring element      
 *   
 * @output_name STRUCTURINGELEMENT
 * @output_description The element structure
 * @output_type CvfdStructuringElement
 *
 END*/
   
   
class CvfdCreateStructuringElement : public BufferedNode {
      
  //Output ID
  int m_structuringElementID;
  //Parameter
  int m_cols;
  int m_rows;
  int m_anchor_x;
  int m_anchor_y;
  string m_shape;
  //map of the shape
  map<string,int> m_shapeMap;
      
public:
  CvfdCreateStructuringElement(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add outputs
    m_structuringElementID = addOutput("STRUCTURINGELEMENT");
         
    //Initialize parameters         
    m_cols = dereference_cast<int>(parameters.get("COLS"));
    m_rows = dereference_cast<int>(parameters.get("ROWS"));
    m_anchor_x = dereference_cast<int>(parameters.get("ANCHOR_X"));
    m_anchor_y = dereference_cast<int>(parameters.get("ANCHORY_y"));
    m_shape = object_cast<String>(parameters.get("SHAPE"));
         
    //Initialize the shape map
    m_shapeMap["CV_SHAPE_RECT"] = CV_SHAPE_RECT;
    m_shapeMap["CV_SHAPE_CROSS"] = CV_SHAPE_CROSS;
    m_shapeMap["CV_SHAPE_ELLIPSE"] = CV_SHAPE_ELLIPSE; 
    m_shapeMap["CV_SHAPE_CUSTOM"] = CV_SHAPE_CUSTOM;
         
  }
            
  void calculate(int output_id, int count, Buffer &out)
  {   
    //Handle
    RCPtr<CvfdStructuringElement> structuringElementPtr(new CvfdStructuringElement(m_cols, m_rows, m_anchor_x, m_anchor_y, m_shapeMap[m_shape]));  
    out[count] = structuringElementPtr;
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdCreateStructuringElement)
};
